(function($) {
    $.fn.annotateImage = function(options) {
	/// <summary>
	///	Creates an annotation on the image
	///	This function shuffles the DIV tags with an annotateImage:
	///	Before:
	///	  <div class="annotateImage">
	///	    <img src="image.gif">
	///	    <div class="image-annotation">
	///	      <div class="image-annotation-title">This is an annotation.</div>
	///	      <div class="image-annotation-region">{"left": 5, "top": 5, "width": 160, "height": 140}</div>
	///	    </div>
	///	  </div>
	///
	///	After:
	///	  <div class="annotateImage">
	///	    <img src="image.gif" style="display: none;">
	///	    <div class="image-annotation-canvas">
	///	      <div class="image-annotations">
	///	        <div class="image-annotation" style="top: 5px; left: 5px; width: 160px; height: 140px; position: absolute; display:none;">
	///	          <div class="image-annotation-title" style="top: 140px; display: none;">
	///                 This is an annotation.
	///	          </div>
	///	        </div>
	///	      </div>
	///	    </div>
	///	  </div>
	/// </summary>
	///
	///
	/// TODO: add checks to see if annotation is already processed

	// Get the 'img'
	this.image = this.children("img");

	// Add the canvas after the 'img'
        this.canvas = $('<div class="image-annotation-canvas"><div class="image-annotations"></div></div>');
	// Get the annotations
        this.canvas.annotations = this.canvas.children('.image-annotations');

	// Set the canvas from image values
	this.canvas.height(this.image.height());
	this.canvas.width(this.image.width());
        this.canvas.css('background-image', 'url("' + this.image.attr('src') + '")');
	this.canvas.hover(function() {
	    var annotations = $(this).children('.image-annotations');
	    $(annotations).children('.image-annotation').show();
	}, function() {
	    var annotations = $(this).children('.image-annotations');
	    $(annotations).children('.image-annotation').hide();
	});

	// Add the canvas after the 'img'
        this.image.after(this.canvas);
	this.image.hide();

	$.fn.annotateImage.processImageAnnotationDivs(this, this.children('.image-annotation'));

	return this;
    };

    $.fn.annotateImage.processImageAnnotationDivs = function(image, annotationdivs) {
	for(var i=0, j=annotationdivs.length; i<j; i++) {
	    var annotationdiv = annotationdivs[i];
	    var note = $.fn.annotateImage.processImageAnnotationDiv(annotationdiv);
	    note = new $.fn.annotateNote(image, note);
	    $(annotationdiv).replaceWith();
	}
    };

    $.fn.annotateImage.processImageAnnotationDiv = function(annotationdiv) {
	/// <summary>
	///	Create a Object with values from 'image-annotation' div.
	///	This parses the JSON text of the 'image-annotation-region' 
	///	sub-div and saves it as the note.region value.
	/// </summary>
	// TODO: error checking
	var title = $(annotationdiv).children('.image-annotation-title');
	var region = $(annotationdiv).children('.image-annotation-region');
	var note = {};

	note.title = $(title).html();

	var coordinates = $(region).text();
	var position    = $.secureEvalJSON(coordinates);
	note.region = position;
	return(note);
    };

    $.fn.annotateNote = function(image, note) {
	/// <summary>
	///	Create an annotation from note[region] and note[title]
	///	note[region] is an object with ints [top, left, width, height]
	///	note[title] is HTML
	/// </summary>
	// TODO: error checking

	this.image = image;
	this.note = note;

	var newannotation = $('<div class="image-annotation"><div class="image-annotation-title"></div></div>');
	var newregion   = $(newannotation).children('.image-annotation');
	var newtitle    = $(newannotation).children('.image-annotation-title');
	var title       = note.title;
	var coords    = note.region;

	$(newannotation).css({
	    top:    coords.top,
	    left:   coords.left,
	    width:  coords.width,
	    height: coords.height,
	    position: 'absolute'
	});
	$(newtitle).html(title);
	$(newtitle).css({
	    top:    coords.height,
	});
	$(newannotation).hover(function() {
	    $(newannotation).children('.image-annotation-title').show();
	},function() {
	    $(newannotation).children('.image-annotation-title').hide();
	});
	this.image.canvas.annotations.append(newannotation);

	$(newtitle).hide();
	$(newannotation).hide();
    }

})(jQuery);
/*
 * TODO: make it auto-magically run on load():
 *
 * $(window).load(function(){
 *  $(".annotated-image").annotateImage({
 *  });
 * });
 */
